<?php

namespace Utopia;

class Route extends Hook
{
    /**
     * HTTP Method
     *
     * @var string
     */
    protected $method = '';

    /**
     * Whether to use hook
     *
     * @var bool
     */
    protected $hook = true;

    /**
     * Path
     *
     * @var string
     */
    protected $path = '';

    /**
     * Path params.
     *
     * @var array<string,string>
     */
    protected $pathParams = [];

    /**
     * Internal counter.
     *
     * @var int
     */
    protected static $counter = 0;

    /**
     * Route order ID.
     *
     * @var int
     */
    protected $order;

    public function __construct($method, string $path)
    {
        $this->path($path);
        $this->method = $method;
        $this->order = ++self::$counter;
        $this->action = function () {
        };
    }

    /**
     * Get Route Order ID
     *
     * @return int
     */
    public function getOrder()
    {
        return $this->order;
    }

    /**
     * Add path
     *
     * @param string $path
     * @return self
     */
    public function path($path)
    {
        $this->path = $path;

        return $this;
    }

    /**
     * Add alias
     *
     * @param  string $path
     * @return self
     */
    public function alias($path)
    {
        Router::addRouteAlias($path, $this);

        return $this;
    }

    /**
     * When set false, hooks for this route will be skipped.
     *
     * @param bool $hook
     * @return self
     */
    public function hook($hook = true)
    {
        $this->hook = $hook;

        return $this;
    }

    /**
     * Get HTTP Method
     *
     * @return string
     */
    public function getMethod()
    {
        return $this->method;
    }

    /**
     * Get path
     *
     * @return string
     */
    public function getPath()
    {
        return $this->path;
    }

    /**
     * Get hook status
     *
     * @return bool
     */
    public function getHook()
    {
        return $this->hook;
    }

    /**
     * Set path param.
     *
     * @param string $key
     * @param int $index
     * @return void
     */
    public function setPathParam($key, $index)
    {
        $this->pathParams[$key] = $index;
    }

    /**
     * Get path params.
     *
     * @param \Utopia\Request $request
     * @return array
     */
    public function getPathValues(Request $request)
    {
        $pathValues = [];
        $parts = explode('/', ltrim($request->getURI(), '/'));

        foreach ($this->pathParams as $key => $index) {
            if (array_key_exists($index, $parts)) {
                $pathValues[$key] = $parts[$index];
            }
        }

        return $pathValues;
    }
}
